
#include "i2c_gpio.h"

void i2c_set_data_high(void)
{
	USIU.SGPIODT2.B.SGPIOC |= 0x02 ;
}

void i2c_set_data_low(void)
{

	USIU.SGPIODT2.B.SGPIOC &= 0xFD ;
}

void i2c_set_clock_high(void)
{

	USIU.SGPIODT2.B.SGPIOC |= 0x01 ;
}

void i2c_set_clock_low(void)
{

	USIU.SGPIODT2.B.SGPIOC &= 0xFE ;
}

unsigned char i2c_get_data(void)
{
	unsigned char result ;
	result = (unsigned char) (USIU.SGPIODT2.B.SGPIOC & 0x02);

	return  (unsigned char) (result>>1) ;
}

void i2c_set_data_output(void)
{
	USIU.SGPIOCR.B.SDDRC |= 0x02 ;
}


void i2c_set_data_input(void)
{
	USIU.SGPIOCR.B.SDDRC &= 0xFD ;
}

void i2c_set_clock_output(void)
{
	USIU.SGPIOCR.B.SDDRC |= 0x01 ;
}


void i2c_delay(void)
{
	unsigned int count ;

	for (count=0; count<2000; count++) ;


}


void i2c_start(void)
{
// Need a high to low transition while clock is high
	i2c_set_data_output() ;

	i2c_set_data_high() ;
	i2c_delay() ;
	i2c_set_clock_high() ;
	i2c_delay() ;
	i2c_set_data_low() ;
	i2c_delay() ;
	i2c_set_clock_low() ;
	i2c_delay() ;
}

void i2c_init(void)
{
	i2c_set_clock_output() ;
	i2c_set_data_output() ;
	USIU.SGPIODT2.B.SGPIOC |= 3 ; // set data and clock high
	i2c_delay() ;


}
void i2c_stop(void)
{
// Need a low to high transition while clock is high
	i2c_set_data_output() ;

	i2c_set_data_low() ;
	i2c_delay() ;
	i2c_set_clock_high() ;
	i2c_delay() ;
	i2c_set_data_high() ;
	i2c_delay() ;
	// SCL and SDA stay high after stop for idle condition
}

void i2c_write(unsigned char output_data)
{
	unsigned char index ;
	i2c_set_data_output() ;

	i2c_set_clock_low() ;

	for (index=0; index<8 ; index++) {
		i2c_delay() ;
		if (output_data & 0x80)
			i2c_set_data_high() ;
		else
			i2c_set_data_low() ;
		i2c_delay() ;
		output_data <<= 1 ;
		i2c_set_clock_high() ;
		i2c_delay() ;
		i2c_set_clock_low() ;
	}
	// put data pin into read mode

	//i2c_set_data_high() ;
	i2c_set_data_input() ;
	i2c_delay() ;
}

unsigned char i2c_read(void)
{
	unsigned char index, input_data ;

	i2c_set_data_input() ;

	input_data = 0x00 ;
	for (index=0; index<8 ; index++) {
		input_data <<= 1 ;
		i2c_delay() ;
		i2c_set_clock_high() ;
		i2c_delay() ;
		input_data |= i2c_get_data() ;
		i2c_set_clock_low() ;
	}

	return input_data ;

}

void i2c_wait_acknowledge(void)
{
	i2c_set_clock_high() ;
	while (i2c_get_data()) ;
	i2c_delay() ;
	i2c_set_clock_low() ;
	i2c_delay() ;
}

void i2c_send_acknowledge(unsigned char ack_bit)
{
	i2c_set_data_output() ;
	if (ack_bit)
		i2c_set_data_high();
	else
		i2c_set_data_low() ;
	i2c_delay() ;
	i2c_set_clock_high() ;
	i2c_delay() ;
}

unsigned char i2c_get_register(unsigned char address, unsigned char reg)
{
	unsigned char seconds=0 ;

	i2c_start() ;
 	i2c_write(address) ;

	i2c_wait_acknowledge() ;

	i2c_write(reg) ; // address of register

	i2c_wait_acknowledge() ;

	i2c_start() ;

 	i2c_write(address | 0x01) ;

	i2c_wait_acknowledge() ;

	seconds = i2c_read() ;

	i2c_send_acknowledge(1) ;

	i2c_stop() ;

	return seconds ;

}



