
#include "i2c_tpu.h"

#include <Motorola/m_common.h>
#include <Motorola/mpc555.h>
#include <Motorola/tpu_dio.h>
#include <Motorola/mpc500_util.h>

void i2c_set_data_high(struct i2c_tpu_data *tpu_data)
{                           
	tpu_dio_output_high(tpu_data->tpu, tpu_data->datapin);
}

void i2c_set_data_low(struct i2c_tpu_data *tpu_data)
{
	tpu_dio_output_low(tpu_data->tpu, tpu_data->datapin);
}

void i2c_set_clock_high(struct i2c_tpu_data *tpu_data)
{
	//clock should always be in output state
	tpu_dio_output_high(tpu_data->tpu, tpu_data->clockpin);
}

void i2c_set_clock_low(struct i2c_tpu_data *tpu_data)
{
	//clock should always be in output state
	tpu_dio_output_low(tpu_data->tpu, tpu_data->clockpin);
}

unsigned char i2c_get_data(struct i2c_tpu_data *tpu_data)
{
	return((tpu_dio_input_immed(tpu_data->tpu, tpu_data->datapin) >> 15) & 1);
}

void i2c_set_data_output(struct i2c_tpu_data *tpu_data)
{
	//default to data high
	tpu_dio_init_output(tpu_data->tpu, tpu_data->datapin, TPU_PRIORITY_HIGH, TPU_DIO_PIN_HIGH);
}

void i2c_set_data_input(struct i2c_tpu_data *tpu_data)  
{                            
	tpu_dio_init_input_immed(tpu_data->tpu, tpu_data->datapin, TPU_PRIORITY_HIGH);
}

void i2c_set_clock_output(struct i2c_tpu_data *tpu_data)
{
	//default to clock high
	tpu_dio_init_output(tpu_data->tpu, tpu_data->clockpin, TPU_PRIORITY_HIGH, TPU_DIO_PIN_HIGH);
}

void i2c_delay(void)
{
        unsigned int count ;

        for (count=0; count<500; count++) ;


}

void i2c_start(struct i2c_tpu_data *tpu_data)
{
// Need a high to low transition while clock is high

        i2c_set_data_output(tpu_data);

        i2c_set_data_high(tpu_data) ;
        i2c_delay() ;
        i2c_set_clock_high(tpu_data) ;
        i2c_delay() ;
        i2c_set_data_low(tpu_data) ;
        i2c_delay() ;
        i2c_set_clock_low(tpu_data) ;
        i2c_delay() ;
}

void i2c_init(struct i2c_tpu_data *tpu_data)
{
        i2c_set_data_output(tpu_data);
        i2c_set_clock_output(tpu_data);
        i2c_delay() ;
	//Need to set both clock and data high
	//Since we can't do it perfectly simultaneously, it seems best to do
	//  the data first, then the clock.  That way, we don't have a
	//  transition of the data line while the clock is high.
	//(Note that since the set to output functions called above already		//  do this, we probably needn't worry!)
	i2c_set_data_high(tpu_data);
	i2c_set_clock_high(tpu_data);
        i2c_delay() ;
}

void i2c_stop(struct i2c_tpu_data *tpu_data)
{
// Need a low to high transition while clock is high
        i2c_set_data_output(tpu_data);

        i2c_set_data_low(tpu_data) ;
        i2c_delay() ;
        i2c_set_clock_high(tpu_data) ;
        i2c_delay() ;
        i2c_set_data_high(tpu_data) ;
        i2c_delay() ;
        // SCL and SDA stay high after stop for idle condition
}

void i2c_write(struct i2c_tpu_data *tpu_data, unsigned char output_data)
{
        unsigned char index ;
        i2c_set_data_output(tpu_data) ;

        i2c_set_clock_low(tpu_data) ;

        for (index=0; index<8 ; index++) {
                i2c_delay() ;
                if (output_data & 0x80)
                        i2c_set_data_high(tpu_data) ;
                else
                        i2c_set_data_low(tpu_data) ;
                i2c_delay() ;
                output_data <<= 1 ;
                i2c_set_clock_high(tpu_data) ;
                i2c_delay() ;
                i2c_set_clock_low(tpu_data) ;
        }
        // put data pin into read mode

        //i2c_set_data_high() ;
        i2c_set_data_input(tpu_data) ;
        i2c_delay() ;
}

unsigned char i2c_read(struct i2c_tpu_data *tpu_data)
{
        unsigned char index, input_data ;

        i2c_set_data_input(tpu_data) ;

        input_data = 0x00 ;
        for (index=0; index<8 ; index++) {
                input_data <<= 1 ;
                i2c_delay() ;
                i2c_set_clock_high(tpu_data) ;
                i2c_delay() ;
                input_data |= i2c_get_data(tpu_data) ;
                i2c_set_clock_low(tpu_data) ;
        }

        return input_data ;

}

void i2c_wait_acknowledge(struct i2c_tpu_data *tpu_data)
{
	i2c_set_data_input(tpu_data);
        i2c_set_clock_high(tpu_data) ;
        while (i2c_get_data(tpu_data)) ;
        i2c_delay() ;
        i2c_set_clock_low(tpu_data) ;
        i2c_delay() ;
}

void i2c_send_acknowledge(struct i2c_tpu_data *tpu_data, unsigned char ack_bit)
{
        i2c_set_data_output(tpu_data) ;
        if (ack_bit)
                i2c_set_data_high(tpu_data);
        else
                i2c_set_data_low(tpu_data) ;
        i2c_delay() ;
        i2c_set_clock_high(tpu_data) ;
        i2c_delay() ;
        i2c_set_clock_low(tpu_data) ;
        i2c_delay() ;
	
}

unsigned char i2c_get_register(struct i2c_tpu_data *tpu_data, unsigned char address, unsigned char reg)
{
        unsigned char result=0 ;

        i2c_start(tpu_data) ;
        i2c_write(tpu_data, address) ;

        i2c_wait_acknowledge(tpu_data) ;

        i2c_write(tpu_data, reg) ; // address of register

        i2c_wait_acknowledge(tpu_data) ;

        i2c_start(tpu_data) ;

        i2c_write(tpu_data, address | 0x01) ;

        i2c_wait_acknowledge(tpu_data) ;

        result = i2c_read(tpu_data) ;

        i2c_send_acknowledge(tpu_data, 1) ;

        i2c_stop(tpu_data) ;

        return result ;

}

unsigned char i2c_set_register(struct i2c_tpu_data *tpu_data, unsigned char address, unsigned char reg, unsigned char value)
{

        i2c_start(tpu_data) ;
        i2c_write(tpu_data, address) ;

        i2c_wait_acknowledge(tpu_data) ;

        i2c_write(tpu_data, reg) ; // address of register

        i2c_wait_acknowledge(tpu_data) ;

        i2c_write(tpu_data, value) ;

        i2c_wait_acknowledge(tpu_data) ;

        i2c_stop(tpu_data) ;

        return 0;
}


